==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/item/equipment/bauble/RingOfMagnetizationItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import vazkii.botania.api.BotaniaAPI;
import vazkii.botania.client.fx.SparkleParticleData;
import vazkii.botania.common.handler.EquipmentHandler;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.common.helper.MathHelper;
import vazkii.botania.common.lib.BotaniaTags;
import vazkii.botania.mixin.ItemEntityAccessor;
import vazkii.botania.xplat.BotaniaConfig;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.List;
import net.minecraft.class_1309;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1542;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2338;
import net.minecraft.class_2378;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_2960;

public class RingOfMagnetizationItem extends BaubleItem {

	private static final String TAG_COOLDOWN = "cooldown";

	private final int range;

	public RingOfMagnetizationItem(class_1793 props) {
		this(props, 6);
	}

	public RingOfMagnetizationItem(class_1793 props, int range) {
		super(props);
		this.range = range;
	}

	public static void onTossItem(class_1657 player) {
		class_1799 ring = EquipmentHandler.findOrEmpty(s -> s.method_7909() instanceof RingOfMagnetizationItem, player);
		if (!ring.method_7960()) {
			setCooldown(ring, 100);
		}
	}

	@Override
	public Multimap<class_1320, class_1322> getEquippedAttributeModifiers(class_1799 stack) {
		if (XplatAbstractions.INSTANCE.isModLoaded("malum")) {
			Multimap<class_1320, class_1322> attributes = HashMultimap.create();
			attributes.put(class_2378.field_23781.method_10223(new class_2960("malum", "spirit_reach")),
					new class_1322(getBaubleUUID(stack), "Magnet Ring reach boost", 0.5, class_1322.class_1323.field_6330));
			return attributes;
		}
		return super.getEquippedAttributeModifiers(stack);
	}

	@Override
	public void onWornTick(class_1799 stack, class_1309 living) {
		super.onWornTick(stack, living);

		if (living.method_7325()) {
			return;
		}

		int cooldown = getCooldown(stack);

		if (BotaniaAPI.instance().hasSolegnoliaAround(living)) {
			if (cooldown < 0) {
				setCooldown(stack, 2);
			}
			return;
		}

		if (cooldown <= 0) {
			if (living.method_5715() == BotaniaConfig.common().invertMagnetRing()) {
				double x = living.method_23317();
				double y = living.method_23318() + 0.75;
				double z = living.method_23321();

				int range = ((RingOfMagnetizationItem) stack.method_7909()).range;
				List<class_1542> items = living.field_6002.method_18467(class_1542.class, new class_238(x - range, y - range, z - range, x + range, y + range, z + range));
				int pulled = 0;
				for (class_1542 item : items) {
					if (((RingOfMagnetizationItem) stack.method_7909()).canPullItem(item)) {
						if (pulled > 200) {
							break;
						}

						MathHelper.setEntityMotionFromVector(item, new class_243(x, y, z), 0.45F);
						if (living.field_6002.field_9236) {
							boolean red = living.field_6002.field_9229.method_43056();
							float r = red ? 1F : 0F;
							float b = red ? 0F : 1F;
							SparkleParticleData data = SparkleParticleData.sparkle(1F, r, 0F, b, 3);
							living.field_6002.method_8406(data, item.method_23317(), item.method_23318(), item.method_23321(), 0, 0, 0);
						}
						pulled++;
					}
				}
			}
		} else {
			setCooldown(stack, cooldown - 1);
		}
	}

	private boolean canPullItem(class_1542 item) {
		int pickupDelay = ((ItemEntityAccessor) item).getPickupDelay();
		if (!item.method_5805() || pickupDelay >= 40
				|| BotaniaAPI.instance().hasSolegnoliaAround(item)
				|| XplatAbstractions.INSTANCE.preventsRemoteMovement(item)) {
			return false;
		}

		class_1799 stack = item.method_6983();
		if (stack.method_7960()
				|| XplatAbstractions.INSTANCE.findManaItem(stack) != null
				|| XplatAbstractions.INSTANCE.findRelic(stack) != null
				|| stack.method_31573(BotaniaTags.Items.MAGNET_RING_BLACKLIST)) {
			return false;
		}

		class_2338 pos = item.method_24515();

		if (item.field_6002.method_8320(pos).method_26164(BotaniaTags.Blocks.MAGNET_RING_BLACKLIST)) {
			return false;
		}

		if (item.field_6002.method_8320(pos.method_10074()).method_26164(BotaniaTags.Blocks.MAGNET_RING_BLACKLIST)) {
			return false;
		}

		return true;
	}

	public static int getCooldown(class_1799 stack) {
		return ItemNBTHelper.getInt(stack, TAG_COOLDOWN, 0);
	}

	public static void setCooldown(class_1799 stack, int cooldown) {
		ItemNBTHelper.setInt(stack, TAG_COOLDOWN, cooldown);
	}
}

==> Botania-1.20.1-451-FABRIC-20251204.072140-31-sources.jar: vazkii/botania/common/item/equipment/bauble/RingOfMagnetizationItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import vazkii.botania.api.BotaniaAPI;
import vazkii.botania.client.fx.SparkleParticleData;
import vazkii.botania.common.handler.EquipmentHandler;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.common.helper.MathHelper;
import vazkii.botania.common.lib.BotaniaTags;
import vazkii.botania.mixin.ItemEntityAccessor;
import vazkii.botania.xplat.BotaniaConfig;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.List;
import net.minecraft.class_1309;
import net.minecraft.class_1542;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2338;
import net.minecraft.class_238;
import net.minecraft.class_243;

public class RingOfMagnetizationItem extends BaubleItem {

	private static final String TAG_COOLDOWN = "cooldown";

	private final int range;

	public RingOfMagnetizationItem(class_1793 props) {
		this(props, 6);
	}

	public RingOfMagnetizationItem(class_1793 props, int range) {
		super(props);
		this.range = range;
	}

	public static void onTossItem(class_1657 player) {
		class_1799 ring = EquipmentHandler.findOrEmpty(s -> s.method_7909() instanceof RingOfMagnetizationItem, player);
		if (!ring.method_7960()) {
			setCooldown(ring, 100);
		}
	}

	@Override
	public void onWornTick(class_1799 stack, class_1309 living) {
		super.onWornTick(stack, living);

		if (living.method_7325()) {
			return;
		}

		int cooldown = getCooldown(stack);

		if (BotaniaAPI.instance().hasSolegnoliaAround(living)) {
			if (cooldown < 0) {
				setCooldown(stack, 2);
			}
			return;
		}

		if (cooldown <= 0) {
			if (living.method_5715() == BotaniaConfig.common().invertMagnetRing()) {
				double x = living.method_23317();
				double y = living.method_23318() + 0.75;
				double z = living.method_23321();

				int range = ((RingOfMagnetizationItem) stack.method_7909()).range;
				List<class_1542> items = living.method_37908().method_18467(class_1542.class, new class_238(x - range, y - range, z - range, x + range, y + range, z + range));
				int pulled = 0;
				for (class_1542 item : items) {
					if (((RingOfMagnetizationItem) stack.method_7909()).canPullItem(item)) {
						if (pulled > 200) {
							break;
						}

						MathHelper.setEntityMotionFromVector(item, new class_243(x, y, z), 0.45F);
						if (living.method_37908().field_9236) {
							boolean red = living.method_37908().field_9229.method_43056();
							float r = red ? 1F : 0F;
							float b = red ? 0F : 1F;
							SparkleParticleData data = SparkleParticleData.sparkle(1F, r, 0F, b, 3);
							living.method_37908().method_8406(data, item.method_23317(), item.method_23318(), item.method_23321(), 0, 0, 0);
						}
						pulled++;
					}
				}
			}
		} else {
			setCooldown(stack, cooldown - 1);
		}
	}

	private boolean canPullItem(class_1542 item) {
		int pickupDelay = ((ItemEntityAccessor) item).getPickupDelay();
		if (!item.method_5805() || pickupDelay >= 40
				|| BotaniaAPI.instance().hasSolegnoliaAround(item)
				|| XplatAbstractions.INSTANCE.preventsRemoteMovement(item)) {
			return false;
		}

		class_1799 stack = item.method_6983();
		if (stack.method_7960()
				|| XplatAbstractions.INSTANCE.findManaItem(stack) != null
				|| XplatAbstractions.INSTANCE.findRelic(stack) != null
				|| stack.method_31573(BotaniaTags.Items.MAGNET_RING_BLACKLIST)) {
			return false;
		}

		class_2338 pos = item.method_24515();

		if (item.method_37908().method_8320(pos).method_26164(BotaniaTags.Blocks.MAGNET_RING_BLACKLIST)) {
			return false;
		}

		if (item.method_37908().method_8320(pos.method_10074()).method_26164(BotaniaTags.Blocks.MAGNET_RING_BLACKLIST)) {
			return false;
		}

		return true;
	}

	public static int getCooldown(class_1799 stack) {
		return ItemNBTHelper.getInt(stack, TAG_COOLDOWN, 0);
	}

	public static void setCooldown(class_1799 stack, int cooldown) {
		ItemNBTHelper.setInt(stack, TAG_COOLDOWN, cooldown);
	}
}
